﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CSharpDemo
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
            string wow6432Node;

            comboBoxAPI.SelectedIndex = 2;
            if (IntPtr.Size == 4)
            {
                wow6432Node = "";
            }
            else
            {
                wow6432Node = "WOW6432Node\\";
            }

            object latestDBT = Microsoft.Win32.Registry.GetValue("HKEY_LOCAL_MACHINE\\SOFTWARE\\" + wow6432Node + "Duxbury Systems, Inc.", "LatestDBT", "");
            string dbtw = latestDBT as string;
            string path = dbtw.Substring(0, dbtw.LastIndexOf("\\"));
            textBoxInstallation.Text = path;
            initializationComplete = true;
            textBoxBTB.Text = "amerc";
            textBoxSCT.Text = "engdxp";
            textBoxInput.Text = "Hello, world!";
            reloadDuxbury();
        }

        private Duxbury duxbury;
        private bool initializationComplete = false;

        private void handleCursorRoutingUpdate()
        {
            Int16 inputSelectionStart = 0;
            switch (comboBoxAPI.SelectedIndex)
            {
                case 0: // ANSI
                case 1: // Unicode
                    inputSelectionStart = duxbury.InputPosFromOutputPos(textBoxInput.Text, (short)textBoxOutput.SelectionStart);
                    break;
                case 2: // Fixed Buffer
                case 3: // Dynamic Buffer
                    inputSelectionStart = duxbury.LocateInputPos((short)textBoxOutput.SelectionStart);
                    break;
            }
            if (inputSelectionStart >= 0)
            {
                textBoxInput.SelectionStart = inputSelectionStart;
                int x = textBoxInput.GetPositionFromCharIndex(inputSelectionStart).X
                    + textBoxInput.Location.X
                    - (labelAboveInput.Width / 2)
                    + 2; // a-priori, +2 is proper tweak for Peter's display.
                labelAboveInput.Location = new Point(x, labelAboveInput.Location.Y);
                labelBelowInput.Location = new Point(x, labelBelowInput.Location.Y);
            }
            // else TODO
        }

        private void linkLabelWebsite_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start("https://www.duxburysystems.com");
        }

        private void translate()
        {
            if (duxbury != null && duxbury.IsOk)
            {
                switch (comboBoxAPI.SelectedIndex)
                {
                    case 0:
                        textBoxOutput.Text = duxbury.TranslateStringA(textBoxInput.Text);
                        break;
                    case 1:
                        textBoxOutput.Text = duxbury.TranslateStringW(textBoxInput.Text);
                        break;
                    case 2:
                        textBoxOutput.Text = duxbury.TranslateStringEx(textBoxInput.Text, 4 * textBoxInput.Text.Length);
                        break;
                    case 3:
                        textBoxOutput.Text = duxbury.TranslateStringEx(textBoxInput.Text);
                        break;
                }
                if (comboBoxAPI.SelectedIndex > 0)
                {
                    textBoxOutput.Text = duxbury.TranslateStringW(textBoxInput.Text);
                }
                else
                {
                    textBoxOutput.Text = duxbury.TranslateStringA(textBoxInput.Text);
                }
            }
        }
        private void textBoxInput_TextChanged(object sender, EventArgs e)
        {
            translate();
        }

        private void textBoxOutput_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Left || e.KeyCode == Keys.Right)
            {
                handleCursorRoutingUpdate();
            }
        }

        private void textBoxOutput_MouseClick(object sender, MouseEventArgs e)
        {
            handleCursorRoutingUpdate();
        }

        private void reloadDuxbury()
        {
            string btb = textBoxBTB.Text;
            string sct = textBoxSCT.Text;

            if (!btb.EndsWith(".btb")) btb += ".btb";
            if (!btb.EndsWith(".sct")) sct += ".sct";

            bool unicode = (comboBoxAPI.SelectedIndex > 0);
            bool in28xx = unicode && checkBox28xxInput.Checked;
            bool out28xx = unicode && checkBox28xxOutput.Checked;

            Duxbury.TablePath = textBoxInstallation.Text;
            duxbury = new Duxbury(btb, sct, unicode, in28xx, out28xx);

            checkBox28xxInput.Enabled = unicode;
            checkBox28xxOutput.Enabled = unicode;
            textBoxInput.Enabled = duxbury.IsOk;
            textBoxOutput.Enabled = duxbury.IsOk;
            labelLastFailure.Text = duxbury.LastFailure;
            labelLastFailure.Visible = !duxbury.IsOk;

            translate();
        }

        private void textBoxBTB_TextChanged(object sender, EventArgs e)
        {
            reloadDuxbury();
        }

        private void textBoxSCT_TextChanged(object sender, EventArgs e)
        {
            reloadDuxbury();
        }

        private void checkBoxUnicode_CheckedChanged(object sender, EventArgs e)
        {
            reloadDuxbury();
        }

        private void checkBox28xxInput_CheckedChanged(object sender, EventArgs e)
        {
            reloadDuxbury();
        }

        private void checkBox28xxOutput_CheckedChanged(object sender, EventArgs e)
        {
            reloadDuxbury();
        }

        private void textBoxInstallation_TextChanged(object sender, EventArgs e)
        {
            reloadDuxbury();
        }

        private void comboBoxAPI_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (initializationComplete)
            {
                reloadDuxbury();
            }
        }
    }
}
